"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TemplateExtractor = void 0;
const fs = __importStar(require("fs"));
const config_1 = require("./_constantes/config");
const path_1 = __importDefault(require("path"));
const ErroMsgs_1 = require("./_constantes/ErroMsgs");
const NamingConvention_1 = require("./_constantes/NamingConvention");
const camel_case_1 = require("camel-case");
const param_case_1 = require("param-case");
const pascal_case_1 = require("pascal-case");
const snake_case_1 = require("snake-case");
const constant_case_1 = require("constant-case");
const sentence_case_1 = require("sentence-case");
const optionsCaseTransform = {
    stripRegexp: new RegExp("[ _-]", "g"),
    splitRegexp: /([a-z])([A-Z0-9])/g
};
const caseTransform = {
    camelCase: (str) => (0, camel_case_1.camelCase)(str, optionsCaseTransform),
    paramCase: (str) => (0, param_case_1.paramCase)(str, optionsCaseTransform),
    pascalCase: (str) => (0, pascal_case_1.pascalCase)(str, optionsCaseTransform),
    snakeCase: (str) => (0, snake_case_1.snakeCase)(str, optionsCaseTransform),
    constantCase: (str) => (0, constant_case_1.constantCase)(str, optionsCaseTransform),
    sentenceCase: (str) => (0, sentence_case_1.sentenceCase)(str, optionsCaseTransform)
};
const regexPath = new RegExp("(?:\\.|\\.\\.)\\/(?:\\.\\.\\/)*[\\w_-]+", "g");
class TemplateExtractor {
    constructor(templateReader, replaceVal, subdomain) {
        this.templateReader = templateReader;
        this.replaceVal = replaceVal;
        this.subdomain = subdomain;
    }
    getComponentDirName() {
        const templateDirName = this.templateReader.templateDirName;
        const convention = this._getConvention(templateDirName);
        let formatedReplaceValue = this.replaceVal;
        if (!convention)
            return templateDirName;
        else if (convention && convention !== "none")
            formatedReplaceValue = caseTransform[convention](formatedReplaceValue);
        return templateDirName.replace(this._buildClassicRegex(convention), formatedReplaceValue);
    }
    getComponentFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            const filesResult = yield this.templateReader.getTemplateFiles();
            if (filesResult instanceof Error)
                return filesResult;
            return filesResult.map(fileInfo => {
                // this._transformPaths(fileInfo.fileData,"src/core/enzoComp")
                const conventionFileName = this._getConvention(fileInfo.fileName);
                let formatedFileName = fileInfo.fileName;
                let formatedReplaceValue = this.replaceVal;
                if (conventionFileName && conventionFileName !== "none")
                    formatedReplaceValue = caseTransform[conventionFileName](formatedReplaceValue);
                if (conventionFileName)
                    formatedFileName = fileInfo.fileName.replace(this._buildClassicRegex(conventionFileName), formatedReplaceValue);
                const formatedData = this._changeFileContent(fileInfo.fileData);
                return {
                    fileName: formatedFileName,
                    data: formatedData
                };
            });
        });
    }
    getComponentWrkDirPath(componentWrkDir) {
        let compWrkDirPath;
        if (typeof componentWrkDir === "string") {
            compWrkDirPath = path_1.default.resolve(componentWrkDir);
            if (!fs.existsSync(compWrkDirPath))
                return new Error(ErroMsgs_1.ErroMsgs.COMPONENT_WRK_DIR_NOT_EXISTE(compWrkDirPath));
            return compWrkDirPath;
        }
        const { rootWorkDir, extensionWorkDir } = componentWrkDir;
        const rootWrkDirPath = path_1.default.resolve(rootWorkDir);
        if (!fs.existsSync(rootWrkDirPath))
            return new Error(ErroMsgs_1.ErroMsgs.ROOT_WRK_DIR_NOT_EXISTE(rootWorkDir));
        let subdomainPath;
        if (this.subdomain) {
            subdomainPath = `${rootWrkDirPath}/${this.subdomain}`;
            if (!fs.existsSync(subdomainPath))
                return new Error(ErroMsgs_1.ErroMsgs.SUBDOMAIN_PATH_NOT_EXISTE(this.subdomain, path_1.default.relative(path_1.default.resolve(), rootWrkDirPath)));
            compWrkDirPath = `${subdomainPath}/${this.subdomain}/${extensionWorkDir}`;
            if (!fs.existsSync(compWrkDirPath))
                return new Error(ErroMsgs_1.ErroMsgs.EXTENTION_WRK_DIR_NOT_EXISTE(extensionWorkDir, path_1.default.relative(path_1.default.resolve(), subdomainPath)));
            return compWrkDirPath;
        }
        compWrkDirPath = `${rootWrkDirPath}/${extensionWorkDir}`;
        if (!fs.existsSync(compWrkDirPath))
            return new Error(ErroMsgs_1.ErroMsgs.EXTENTION_WRK_DIR_NOT_EXISTE(extensionWorkDir, path_1.default.relative(path_1.default.resolve(), subdomainPath)));
        return compWrkDirPath;
    }
    _changeFileContent(data) {
        if (!data)
            return "";
        const conventions = this._getAllConventions(data);
        if (!conventions.length)
            return data;
        const superRegex = this._buildSuperRegex(conventions);
        return data.replace(superRegex, generic => {
            const convention = this._getConvention(generic);
            let formatedReplaceValue = this.replaceVal;
            if (convention !== "none")
                formatedReplaceValue = caseTransform[convention](formatedReplaceValue);
            return formatedReplaceValue;
        });
    }
    _getConvention(str) {
        let nameConvention = null;
        for (const namingConv of Object.values(NamingConvention_1.NamingConvention)) {
            if (str.includes(`${config_1.edging}${namingConv}${config_1.edging}`)) {
                nameConvention = namingConv;
                break;
            }
        }
        return nameConvention;
    }
    _getAllConventions(text) {
        let conventions = [];
        for (const namingConv of Object.values(NamingConvention_1.NamingConvention)) {
            if (text.includes(`${config_1.edging}${namingConv}${config_1.edging}`))
                conventions.push(namingConv);
        }
        return conventions;
    }
    _buildClassicRegex(convention) {
        return new RegExp(`\\${config_1.edging}${convention}\\${config_1.edging}`, "g"); //Capture only generic keyword
    }
    _buildSuperRegex(conventions) {
        const preRegexArray = conventions.map(conv => `\\${config_1.edging}${conv}\\${config_1.edging}`);
        return new RegExp(preRegexArray.join("|"), "g");
    }
}
exports.TemplateExtractor = TemplateExtractor;
//# sourceMappingURL=TemplateExtractor.js.map