"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializer = void 0;
const fs_1 = __importDefault(require("fs"));
const config_1 = require("../_constantes/config");
const path_1 = __importDefault(require("path"));
const promises_1 = require("fs/promises");
const TemplateReader_1 = require("../TemplateReader");
const TemplateRepository_1 = require("../TemplateRepository");
const FileHandler_1 = require("../FileHandler");
const ErroMsgs_1 = require("../_constantes/ErroMsgs");
const InfoMsgs_1 = require("../_constantes/InfoMsgs");
class Initializer {
    constructor() {
    }
    reload(showMsg = false) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const configDirPath = path_1.default.resolve(config_1.nameConfigDir);
                yield Initializer.initConfigDir(configDirPath);
                const result = yield (0, promises_1.readdir)(configDirPath, { withFileTypes: true });
                const templates = result
                    .filter(dirent => dirent.isDirectory())
                    .map(temp => ({ dirName: temp.name, dirPath: `${configDirPath}/${temp.name}` }));
                TemplateRepository_1.TemplateRepo.clear();
                const errors = [];
                templates.forEach(templateDirInfo => {
                    const initTemplateResult = Initializer.initTemplate(templateDirInfo);
                    if (initTemplateResult instanceof Error)
                        return errors.push(initTemplateResult.message);
                });
                if (errors.length)
                    return new Error(errors.join("\n"));
                if (showMsg)
                    console.log(InfoMsgs_1.InfoMsgs.SUCCESS_INITIALIZATION);
                else
                    console.log(InfoMsgs_1.InfoMsgs.SUCCESS_RELOAD);
            }
            catch (err) {
                console.log(ErroMsgs_1.ErroMsgs.UNEXPECTED_ERROR(err));
            }
        });
    }
    static initConfigDir(mkdirPath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (fs_1.default.existsSync(mkdirPath))
                return;
            fs_1.default.mkdirSync(mkdirPath);
            yield this.initExempleTemplate();
        });
    }
    static initTemplate(templateDirInfo) {
        const configFileResult = TemplateReader_1.TemplateReader.getTemplateConfigFile(templateDirInfo.dirPath);
        if (configFileResult instanceof Error)
            return configFileResult;
        const checkingResult = TemplateReader_1.TemplateReader.checkConfigFileProperties(configFileResult);
        if (checkingResult instanceof Error)
            return checkingResult;
        TemplateRepository_1.TemplateRepo.saveTemplate(configFileResult.template, templateDirInfo.dirPath);
    }
    static initExempleTemplate() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const configDirPath = path_1.default.resolve(config_1.nameConfigDir);
                const exampleTemplate = {
                    dirName: `Exemple${config_1.keywordExemple}Template`,
                    files: [
                        { fileName: `My${config_1.keywordExemple}Exemple.jsx`, data: "" },
                        { fileName: `${config_1.nameConfigFile}`, data: `
{
  "template" : "exemple",
  "componentWorkDir" :"src/core"
}
              `
                        },
                        { fileName: `The${config_1.keywordExemple}Amazing.css`, data: "" },
                        { fileName: `OtherExemple${config_1.keywordExemple}.test.ts`, data: "" },
                        { fileName: `LastOne${config_1.keywordExemple}.php`, data: "" },
                    ]
                };
                yield (0, FileHandler_1.fileHandler)(configDirPath, exampleTemplate.dirName, exampleTemplate.files);
                console.log(InfoMsgs_1.InfoMsgs.SUCCESS_EXAMPLE_TEMPLATE_CREATON);
            }
            catch (err) {
                console.log(ErroMsgs_1.ErroMsgs.EXAMPLE_TEMPLATE_CREATION_UNEXPECTRED_ERR);
            }
        });
    }
}
exports.initializer = new Initializer();
//# sourceMappingURL=Initialize.js.map